@@ -1,15 +1,17 @@
+import functools
 import itertools
 import traceback
-from pathlib import Path
 import warnings
+from pathlib import Path
 from queue import Queue
+from typing import Callable
 
 import pandas as pd
 from PyQt5 import QtCore, QtWidgets, QtGui
 
 from rnalysis import __version__
 from rnalysis.gui import gui_style, gui_widgets
-from rnalysis.utils import settings, io
+from rnalysis.utils import settings, io, generic
 
 
 class CheckableFileSystemModel(QtWidgets.QFileSystemModel):
@@ -333,7 +335,7 @@ class GeneSetView(DataView):
         super().__init__(data, name, parent)
         self.label = QtWidgets.QLabel(f"Gene set '{name}': {len(self.data)} features")
 
-        self.data_view = QtWidgets.QListWidget()
+        self.data_view = gui_widgets.ReactiveListWidget()
         self.save_button = QtWidgets.QPushButton('Save gene set', self)
 
         self.init_ui()
@@ -361,7 +363,7 @@ class DataFrameView(DataView):
             shape = (shape[0], 1)
         self.label = QtWidgets.QLabel(f"Table '{name}': {shape[0]} rows, {shape[1]} columns")
 
-        self.data_view = QtWidgets.QTableView()
+        self.data_view = gui_widgets.ReactiveTableView()
         self.save_button = QtWidgets.QPushButton('Save table', self)
 
         self.init_ui()
@@ -711,3 +713,165 @@ def splash_screen():
     splash.showMessage(f"<i>RNAlysis</i> version {__version__}", QtCore.Qt.AlignBottom | QtCore.Qt.AlignHCenter)
     splash.show()
     return splash
+
+
+class FuncExternalWindow(gui_widgets.MinMaxDialog):
+    IGNORED_WIDGETS = {'help_link'}
+    paramsAccepted = QtCore.pyqtSignal(list, dict, object)
+    geneSetsRequested = QtCore.pyqtSignal(object)
+    __slots__ = {'func_name': 'name of the function to be applied',
+                 'func': 'function to be applied',
+                 'signature': 'signature of the function',
+                 'desc': 'description of the function',
+                 'param_desc': 'description of the function parameters',
+                 'help_link': 'link to the documentation page of the function',
+                 'excluded_params': 'parameters to be excluded from the window',
+                 'scroll': 'scroll area',
+                 'scroll_widget': 'widget containing the scroll area',
+                 'scroll_layout': 'layout for the scroll widget',
+                 'param_group': 'widget group for the parameter widgets',
+                 'param_grid': 'layout for the parameter widgets',
+                 'param_widgets': 'parameter widgets',
+                 'start_button': 'start button',
+                 'close_button': 'close button'}
+
+    def __init__(self, func_name: str, func: Callable, help_link: str, excluded_params: set, threaded=True,
+                 parent=None):
+        super().__init__(parent)
+        self.func_name = func_name
+        self.func = func
+        self.signature = generic.get_method_signature(self.func)
+        self.desc, self.param_desc = io.get_method_docstring(self.func)
+        self.help_link = help_link
+        self.excluded_params = excluded_params.copy()
+        self.threaded = threaded
+
+        self.widgets = {}
+        self.main_layout = QtWidgets.QVBoxLayout(self)
+
+        self.scroll = QtWidgets.QScrollArea()
+        self.scroll_widget = QtWidgets.QWidget(self.scroll)
+        self.scroll_layout = QtWidgets.QGridLayout(self.scroll_widget)
+
+        self.param_group = QtWidgets.QGroupBox(f"1. Set {func_name} parameters")
+        self.param_grid = QtWidgets.QGridLayout(self.param_group)
+        self.param_widgets = {}
+
+        self.start_button = QtWidgets.QPushButton(f'Start {self.func_name}')
+        self.close_button = QtWidgets.QPushButton('Close')
+
+    def init_ui(self):
+        self.scroll.setHorizontalScrollBarPolicy(QtCore.Qt.ScrollBarAlwaysOff)
+        self.scroll.setWidgetResizable(True)
+        self.scroll.setWidget(self.scroll_widget)
+        self.scroll_layout.setSizeConstraint(QtWidgets.QLayout.SetMinAndMaxSize)
+
+        self.main_layout.addWidget(self.scroll)
+
+        self.param_widgets['help_link'] = QtWidgets.QLabel(
+            text=f'<a href="{self.help_link}">Open documentation for <b>{self.func_name}</b></a>')
+        self.param_widgets['help_link'].setOpenExternalLinks(True)
+        self.main_layout.addWidget(self.param_widgets['help_link'])
+
+        self.scroll_layout.addWidget(self.param_group, 0, 0)
+        self.scroll_layout.addWidget(self.start_button, 1, 0, 1, 2)
+        self.scroll_layout.addWidget(self.close_button, 2, 0, 1, 2)
+
+        if self.threaded:
+            self.start_button.clicked.connect(self.run_function_threaded)
+        else:
+            self.start_button.clicked.connect(self.run_function_in_main_loop)
+
+        self.close_button.clicked.connect(self.close)
+
+        self.init_param_ui()
+
+    def init_param_ui(self):
+        i = 0
+
+        for name, param in self.signature.items():
+            if name in self.excluded_params:
+                continue
+            this_desc = self.param_desc.get(name, '')
+            self.param_widgets[name] = gui_widgets.param_to_widget(param, name)
+            self.connect_widget(self.param_widgets[name])
+
+            label = QtWidgets.QLabel(f'{name}:', self.param_widgets[name])
+            label.setToolTip(this_desc)
+            help_button = gui_widgets.HelpButton()
+            self.param_grid.addWidget(help_button, i, 2)
+            self.param_grid.addWidget(label, i, 0)
+            self.param_grid.addWidget(self.param_widgets[name], i, 1)
+            help_button.connect_param_help(name, this_desc)
+            i += 1
+
+        self.param_grid.setRowStretch(i, 1)
+
+    def connect_widget(self, widget: QtWidgets.QWidget):
+        if isinstance(widget, (gui_widgets.ComboBoxOrOtherWidget, gui_widgets.OptionalWidget)):
+            self.connect_widget(widget.other)
+        elif isinstance(widget, gui_widgets.GeneSetComboBox):
+            widget.boxOpened.connect(functools.partial(self.geneSetsRequested.emit, widget))
+
+    def get_analysis_kwargs(self):
+        kwargs = {}
+        for param_name, widget in self.param_widgets.items():
+            if param_name in self.IGNORED_WIDGETS:
+                continue
+            val = gui_widgets.get_val_from_widget(widget)
+            kwargs[param_name] = val
+        return kwargs
+
+    def get_analysis_args(self):
+        args = []
+        return args
+
+    def run_function_threaded(self):
+        args = self.get_analysis_args()
+        kwargs = self.get_analysis_kwargs()
+        try:
+            self.paramsAccepted.emit(args, kwargs, self.showNormal)
+            self.showMinimized()
+        except Exception as e:
+            self.showNormal()
+            raise e
+
+    def run_function_in_main_loop(self):
+        args = self.get_analysis_args()
+        kwargs = self.get_analysis_kwargs()
+        self.func(*args, **kwargs)
+
+
+class PairedFuncExternalWindow(FuncExternalWindow):
+    __slots__ = {'pairs_group': 'widget group for picking file pairs',
+                 'pairs_grid': 'layout for widget group',
+                 'pairs_widgets': 'widgets for picking file pairs'}
+    EXCLUDED_PARAMS = {'r1_files', 'r2_files'}
+
+    def __init__(self, func_name: str, func: Callable, help_link: str, excluded_params: set, parent=None):
+        super().__init__(func_name, func, help_link, excluded_params, parent=parent)
+
+        self.pairs_group = QtWidgets.QGroupBox("2. Choose FASTQ file pairs")
+        self.pairs_grid = QtWidgets.QGridLayout(self.pairs_group)
+        self.pairs_widgets = {}
+
+    def init_ui(self):
+        self.scroll_layout.addWidget(self.pairs_group, 0, 1)
+        self.setMinimumSize(1250, 650)
+        super().init_ui()
+        self.init_pairs_ui()
+
+    def init_pairs_ui(self):
+        self.pairs_widgets['r1_list'] = gui_widgets.OrderedFileList(self)
+        self.pairs_widgets['r2_list'] = gui_widgets.OrderedFileList(self)
+
+        self.pairs_grid.addWidget(self.pairs_widgets['r1_list'], 1, 0)
+        self.pairs_grid.addWidget(self.pairs_widgets['r2_list'], 1, 1)
+        self.pairs_grid.addWidget(QtWidgets.QLabel("<b>R1 files:</b>"), 0, 0)
+        self.pairs_grid.addWidget(QtWidgets.QLabel("<b>R2 files:</b>"), 0, 1)
+
+    def get_analysis_kwargs(self):
+        kwargs = super().get_analysis_kwargs()
+        kwargs['r1_files'] = self.pairs_widgets['r1_list'].get_sorted_names()
+        kwargs['r2_files'] = self.pairs_widgets['r2_list'].get_sorted_names()
+        return kwargs
