@@ -1,27 +1,26 @@
 import collections
-import io as builtin_io
 import itertools
 import logging
-import queue
 import sys
 import warnings
 from functools import lru_cache
 from pathlib import Path
 from typing import Iterable, List, Tuple, Union, Collection, Set, Dict
 
-import graphviz
 import joblib
-import matplotlib.image as mpimg
 import matplotlib.pyplot as plt
 import numpy as np
 import pandas as pd
 import statsmodels.stats.multitest as multitest
 from matplotlib.cm import ScalarMappable
+from mpl_toolkits.axes_grid1 import make_axes_locatable
 from scipy.stats import hypergeom, ttest_1samp, fisher_exact
 from statsmodels.stats.descriptivestats import sign_test
 from tqdm.auto import tqdm
+from typing_extensions import Literal
 
 from rnalysis.utils import ontology, io, parsing, settings, validation, generic
+from rnalysis.utils.param_typing import PARALLEL_BACKENDS, GRAPHVIZ_FORMATS
 
 try:
     import xlmhg
@@ -54,6 +53,9 @@ XLMHG_SUPPORTED = does_python_version_support_single_set()
 
 
 class EnrichmentRunner:
+    ENRICHMENT_SCORE_YLABEL = r"$\log_2$(Fold Enrichment)"
+    SINGLE_SET_ENRICHMENT_SCORE_YLABEL = r"$\log_2$(XL-mHG enrichment score)"
+
     __slots__ = {'results': 'DataFrame containing enrichment analysis results',
                  'annotation_df': 'DataFrame containing all annotation data per gene',
                  'gene_set': 'the set of genes/genomic features whose enrichment to calculate',
@@ -67,7 +69,8 @@ class EnrichmentRunner:
                  'return_fig': 'indicates whether to return a matplotlib Figure after plotting the results',
                  'plot_horizontal': 'indicates whether to plot the results horizontally or vertically',
                  'set_name': 'name of the set of genes/genomic features whose enrichment is calculated',
-                 'parallel': 'indicates whether to calculate the enrichment using parallel processing',
+                 'parallel_backend': 'indicates whether to use parallel processing, and with which backend',
+                 'exclude_unannotated_genes': 'indicates whether to discard unannotated genes from the background and enrichment sets',
                  'enrichment_func': 'the function to be used to calculate enrichment p-values',
                  'pvalue_kwargs': 'key-worded arguments for the function which calculates enrichment p-values',
                  'single_set': 'indicates whether enrichment is calculated on a single set of genes '
@@ -83,9 +86,10 @@ class EnrichmentRunner:
 
     def __init__(self, genes: Union[set, np.ndarray], attributes: Union[Iterable, str, int], alpha: float,
                  attr_ref_path: str, return_nonsignificant: bool, save_csv: bool, fname: str, return_fig: bool,
-                 plot_horizontal: bool, set_name: str, parallel: bool, enrichment_func_name: str, biotypes=None,
-                 background_set: set = None, biotype_ref_path: str = None, single_set: bool = False,
-                 random_seed: int = None, **pvalue_kwargs):
+                 plot_horizontal: bool, set_name: str,
+                 parallel_backend: Literal[PARALLEL_BACKENDS], enrichment_func_name: str, biotypes=None,
+                 background_set: set = None, biotype_ref_path: str = None, exclude_unannotated_genes: bool = True,
+                 single_set: bool = False, random_seed: int = None, **pvalue_kwargs):
         self.results: pd.DataFrame = pd.DataFrame()
         self.annotation_df: pd.DataFrame = pd.DataFrame()
         self.gene_set = parsing.data_to_set(genes)
@@ -104,11 +108,12 @@ class EnrichmentRunner:
         self.return_fig = return_fig
         self.plot_horizontal = plot_horizontal
         self.set_name = set_name
-        self.parallel = parallel
+        self.parallel_backend = parallel_backend
         self.enrichment_func = self._get_enrichment_func(enrichment_func_name)
         if not self.enrichment_func:
             return
         self.pvalue_kwargs = pvalue_kwargs
+        self.exclude_unannotated_genes = exclude_unannotated_genes
         self.single_set = single_set
         if self.single_set:
             assert biotypes is None, "Enrichment in single_set mode does not accept a 'biotypes' argument."
@@ -192,12 +197,23 @@ class EnrichmentRunner:
             self._get_background_set_from_set()
 
         orig_bg_size = len(self.background_set)
-        self.background_set = self.background_set.intersection(self.annotation_df.index)
+        if self.exclude_unannotated_genes:
+            self.background_set = self.background_set.intersection(self.annotation_df.index)
+        else:
+            self.background_set = self.background_set.union(self.annotation_df.index)
+            ann_missing = self.background_set.difference(self.annotation_df.index)
+            self.annotation_df[ann_missing] = np.nan
+
         if orig_bg_size - len(self.background_set) > 0:
-            warnings.warn(
-                f"{orig_bg_size - len(self.background_set)} genes out of the requested {orig_bg_size} background genes "
-                f"do not {self.printout_params}, and are therefore ignored."
-                f" \nThis leaves a total of {len(self.background_set)} background genes.")
+            warning = f"{orig_bg_size - len(self.background_set)} genes out of the requested {orig_bg_size} " \
+                      f"background genes do not {self.printout_params}"
+            if self.exclude_unannotated_genes:
+                warning += f", and are therefore ignored. " \
+                           f"\nThis leaves a total of {len(self.background_set)} background genes."
+            else:
+                warning += ". \nThese genes are not discarded from the background set because you set the paremeter " \
+                           "'exclude_unannotated_genes' to False. Note that this is not a recommended practice. "
+            warnings.warn(warning)
 
         print(f"{len(self.background_set)} background genes are used.")
 
@@ -403,9 +419,11 @@ class EnrichmentRunner:
             not_in_bg = len(self.gene_set) - len(updated_gene_set)
             self.gene_set = updated_gene_set
             if not_in_bg > 0:
-                warnings.warn(f"{not_in_bg} genes in the enrichment set do not {self.printout_params} "
-                              f"and/or do not appear in the background gene set. \n"
-                              f"Enrichment will be computed on the remaining {len(self.gene_set)} genes.")
+                warning = f"{not_in_bg} genes in the enrichment set do not appear in the background gene set"
+                if self.exclude_unannotated_genes:
+                    warning += f" and/or do not {self.printout_params}"
+                warning += f". \nEnrichment will be computed on the remaining {len(self.gene_set)} genes."
+                warnings.warn(warning)
 
     def fetch_annotations(self):
         self.annotation_df = io.load_csv(self.attr_ref_path)
@@ -447,7 +465,7 @@ class EnrichmentRunner:
 
     def calculate_enrichment(self) -> list:
         self.set_random_seed()
-        if self.parallel:
+        if self.parallel_backend != 'sequential' and len(self.attributes) > 5:
             return self._calculate_enrichment_parallel()
         return self._calculate_enrichment_serial()
 
@@ -466,7 +484,8 @@ class EnrichmentRunner:
         return result
 
     def _calculate_enrichment_parallel(self) -> list:
-        result = generic.ProgressParallel(n_jobs=-1, desc="Calculating enrichment", unit='attribute')(
+        result = generic.ProgressParallel(desc="Calculating enrichment", unit='attribute', n_jobs=-1,
+                                          backend=self.parallel_backend)(
             joblib.delayed(self.enrichment_func)(attribute, **self.pvalue_kwargs) for attribute in self.attributes)
         return result
 
@@ -491,11 +510,11 @@ class EnrichmentRunner:
 
     def plot_results(self) -> plt.Figure:
         if self.single_set:
-            return self.enrichment_bar_plot(ylabel=r"$\log_2$(XL-mHG enrichment score)",
+            return self.enrichment_bar_plot(ylabel=self.SINGLE_SET_ENRICHMENT_SCORE_YLABEL,
                                             title=f"Single-list enrichment for {self.set_name}")
-        return self.enrichment_bar_plot(title=f"Enrichment for {self.set_name}")
+        return self.enrichment_bar_plot(ylabel=self.ENRICHMENT_SCORE_YLABEL, title=f"Enrichment for {self.set_name}")
 
-    def enrichment_bar_plot(self, n_bars: int = 'all', name_col: str = None, center_bars: bool = True,
+    def enrichment_bar_plot(self, n_bars: int = 'all', center_bars: bool = True,
                             ylabel: str = r"$\log_2$(Fold Enrichment)",
                             title: str = 'Enrichment results') -> plt.Figure:
 
@@ -508,8 +527,6 @@ class EnrichmentRunner:
         :type title:
         :param n_bars:
         :type n_bars:
-        :param name_col:
-        :type name_col:
         :param ylabel: plot ylabel.
         :type ylabel: str
         :param center_bars: if True, centers the bars around Y=0. Otherwise, ylim is determined by min/max values.
@@ -527,7 +544,10 @@ class EnrichmentRunner:
         else:
             results = self.results
         # pull names/scores/pvals out to avoid accidentally changing the results DataFrame in-place
-        enrichment_names = results.index.values.tolist() if name_col is None else results[name_col].values.tolist()
+        if 'name' in results.columns:
+            enrichment_names = results['name'].values.tolist()
+        else:
+            enrichment_names = results.index.values.tolist()
         enrichment_scores = results[self.en_score_col].values.tolist()
         enrichment_pvalue = results['padj'].values.tolist()
 
@@ -536,7 +556,8 @@ class EnrichmentRunner:
             figsize = [10.5, 0.4 * (4.8 + self.results.shape[0])]
             bar_func = plt.Axes.barh
             line_func = plt.Axes.axvline
-            cbar_kwargs = dict(location='bottom')
+            cbar_location = 'bottom'
+            cbar_orientation = 'horizontal'
             tick_func = plt.Axes.set_yticks
             ticklabels_func = plt.Axes.set_yticklabels
             ticklabels_kwargs = dict(fontsize=13, rotation=0)
@@ -546,22 +567,26 @@ class EnrichmentRunner:
             figsize = [0.5 * (4.8 + self.results.shape[0]), 4.2]
             bar_func = plt.Axes.bar
             line_func = plt.Axes.axhline
-            cbar_kwargs = dict(location='left')
+            cbar_location = 'left'
+            cbar_orientation = 'vertical'
             tick_func = plt.Axes.set_xticks
             ticklabels_func = plt.Axes.set_xticklabels
             ticklabels_kwargs = dict(fontsize=13, rotation=45)
 
         # set enrichment scores which are 'inf' or '-inf' to be the second highest/lowest enrichment score in the list
-        scores_no_inf = [i for i in enrichment_scores if i != np.inf and i != -np.inf and i < 0]
+        scores_no_inf = [abs(score) for score in enrichment_scores if score != np.inf and score != -np.inf]
         if len(scores_no_inf) == 0:
-            scores_no_inf.append(-1)
+            scores_no_inf.append(3)
+
         for i in range(len(enrichment_scores)):
             if enrichment_scores[i] == -np.inf:
-                enrichment_scores[i] = min(scores_no_inf)
-        if len(enrichment_scores) > 3:
-            max_score = max(np.max(np.abs(enrichment_scores)), 2)
+                enrichment_scores[i] = -max(scores_no_inf)
+            elif enrichment_scores[i] == np.inf:
+                enrichment_scores[i] = max(scores_no_inf)
+        if len(scores_no_inf) > 0:
+            max_score = max(max(scores_no_inf), 3)
         else:
-            max_score = 2
+            max_score = 3
 
         # get color values for bars
         data_color_norm = [0.5 * (1 + i / (np.floor(max_score) + 1)) * 255 for i in enrichment_scores]
@@ -571,7 +596,7 @@ class EnrichmentRunner:
         colors = my_cmap(data_color_norm_8bit)
 
         # generate bar plot
-        fig, ax = plt.subplots(constrained_layout=True, figsize=figsize)
+        fig, ax = plt.subplots(tight_layout=True, figsize=figsize)
         bar = bar_func(ax, range(len(enrichment_names)), enrichment_scores, color=colors, edgecolor='black',
                        linewidth=1, zorder=2)
         bar.tick_labels = enrichment_names
@@ -583,13 +608,6 @@ class EnrichmentRunner:
             linewidth = 1 if ind == 0 else 0.5
             linestyle = '-' if ind == 0 else '-.'
             line_func(ax, ind, color=color, linewidth=linewidth, linestyle=linestyle, zorder=0)
-        # add colorbar
-        sm = ScalarMappable(cmap=my_cmap, norm=plt.Normalize(*bounds))
-        sm.set_array(np.array([]))
-        cbar_label_kwargs = dict(label=ylabel, fontsize=16, labelpad=15)
-        cbar = fig.colorbar(sm, ticks=range(int(bounds[0]), int(bounds[1]) + 1), **cbar_kwargs)
-        cbar.set_label(**cbar_label_kwargs)
-        cbar.ax.tick_params(labelsize=14, pad=6)
         # apply xticks
         tick_func(ax, range(len(enrichment_names)))
         ticklabels_func(ax, enrichment_names, **ticklabels_kwargs)
@@ -623,6 +641,19 @@ class EnrichmentRunner:
             else:
                 ax.set_ybound(bounds)
                 plt.tick_params(axis='y', which='both', left=False, right=False, labelleft=False)
+        # add colorbar
+        divider = make_axes_locatable(ax)
+        cax = divider.append_axes(cbar_location, size=f"{5 * (1 + self.plot_horizontal)}%", pad=0.05)
+        sm = ScalarMappable(cmap=my_cmap, norm=plt.Normalize(*bounds))
+        sm.set_array(np.array([]))
+        cbar_label_kwargs = dict(label=ylabel, fontsize=16, labelpad=15, )
+        cbar = fig.colorbar(sm, ticks=range(int(bounds[0]), int(bounds[1]) + 1), cax=cax, orientation=cbar_orientation)
+        cbar.set_label(**cbar_label_kwargs)
+        cax.tick_params(labelsize=14, pad=6)
+
+        if not self.plot_horizontal:
+            cax.yaxis.set_ticks_position('left')
+            cax.yaxis.set_label_position('left')
 
         plt.show()
         return fig
@@ -655,7 +686,7 @@ class NonCategoricalEnrichmentRunner(EnrichmentRunner):
     def __init__(self, genes: set, attributes: Union[Iterable, str, int], alpha: float, biotypes, background_set: set,
                  attr_ref_path: str, biotype_ref_path: str, save_csv: bool, fname: str,
                  return_fig: bool, plot_log_scale: bool, plot_style: str, n_bins: int, set_name: str,
-                 parallel: bool, parametric_test: bool):
+                 parallel_backend: Literal[PARALLEL_BACKENDS], parametric_test: bool):
 
         assert isinstance(plot_log_scale, bool), f"Invalid type for 'plot_log_scale': '{plot_log_scale}'."
         assert plot_style in {'interleaved', 'overlap'}, f"Invalid value for 'plot_style': '{plot_style}'."
@@ -668,8 +699,8 @@ class NonCategoricalEnrichmentRunner(EnrichmentRunner):
         self.plot_style = plot_style
         self.n_bins = n_bins
         super().__init__(genes, attributes, alpha, attr_ref_path, True, save_csv, fname, return_fig, True, set_name,
-                         parallel,
-                         enrichment_func_name, biotypes, background_set, biotype_ref_path, single_set=False)
+                         parallel_backend, enrichment_func_name, biotypes, background_set, biotype_ref_path,
+                         single_set=False, exclude_unannotated_genes=True)
 
     def _get_enrichment_func(self, pval_func_name: str):
         assert isinstance(pval_func_name, str), f"Invalid type for 'pval_func_name': {type(pval_func_name)}."
@@ -784,23 +815,32 @@ class KEGGEnrichmentRunner(EnrichmentRunner):
                  'return_nonsignificant': 'indicates whether to return results which were not found to be '
                                           'statistically significant after enrichment analysis',
                  'attributes_set': 'set of the attributes/KEGG Pathways for which enrichment should be calculated',
-                 'pathway_names_dict': 'a dict with KEGG Pathway IDs as keys and their names as values'}
+                 'pathway_names_dict': 'a dict with KEGG Pathway IDs as keys and their names as values',
+                 'plot_pathway_graphs': 'indicates whether to plot pathway graphs of the statistically significant KEGG pathways',
+                 'pathway_graphs_format': 'file format for the generated pathway graphs',
+                 'gene_id_translator': 'gene ID translator from KEGG into a user-defined gene ID type'
+                 }
     KEGG_DF_QUERIES = {}
     printout_params = "have any KEGG Annotations asocciated with them"
 
     def __init__(self, genes: Union[set, np.ndarray], organism: Union[str, int], gene_id_type: str, alpha: float,
                  return_nonsignificant: bool, save_csv: bool, fname: str, return_fig: bool, plot_horizontal: bool,
-                 set_name: str, parallel: bool, enrichment_func_name: str, biotypes=None,
-                 background_set: set = None, biotype_ref_path: str = None, single_set: bool = False,
-                 random_seed: int = None, **pvalue_kwargs):
+                 plot_pathway_graphs: bool, set_name: str,
+                 parallel_backend: Literal[PARALLEL_BACKENDS], enrichment_func_name: str,
+                 biotypes=None, background_set: set = None, biotype_ref_path: str = None,
+                 exclude_unannotated_genes: bool = True, single_set: bool = False, random_seed: int = None,
+                 pathway_graphs_format: Literal[GRAPHVIZ_FORMATS] = 'none', **pvalue_kwargs):
         super().__init__(genes, [], alpha, '', return_nonsignificant, save_csv, fname, return_fig, plot_horizontal,
-                         set_name, parallel, enrichment_func_name, biotypes, background_set, biotype_ref_path,
-                         single_set, random_seed, **pvalue_kwargs)
+                         set_name, parallel_backend, enrichment_func_name, biotypes, background_set, biotype_ref_path,
+                         exclude_unannotated_genes, single_set, random_seed, **pvalue_kwargs)
         if not self.enrichment_func:
             return
         self.gene_id_type = gene_id_type
         self.taxon_id, self.organism = self.get_taxon_id(organism)
         self.pathway_names_dict: dict = {}
+        self.plot_pathway_graphs = plot_pathway_graphs
+        self.pathway_graphs_format = pathway_graphs_format
+        self.gene_id_translator = None
 
     def get_taxon_id(self, organism: str):
         if isinstance(organism, str) and organism.lower() == 'auto':
@@ -887,7 +927,9 @@ class KEGGEnrichmentRunner(EnrichmentRunner):
             translator, _, self.gene_id_type = io.find_best_gene_mapping(
                 parsing.data_to_tuple(sparse_annotation_dict.keys()), (source,), None)
         else:
-            translator = io.map_gene_ids(parsing.data_to_list(sparse_annotation_dict.keys()), source, self.gene_id_type)
+            translator = io.map_gene_ids(parsing.data_to_tuple(sparse_annotation_dict.keys()), source,
+                                         self.gene_id_type)
+        self.gene_id_translator = translator
         for gene_id in sparse_annotation_dict:
             if gene_id in translator:
                 translated_sparse_annotation_dict[translator[gene_id]] = sparse_dict_cp.pop(gene_id)
@@ -900,6 +942,28 @@ class KEGGEnrichmentRunner(EnrichmentRunner):
         self.attributes = parsing.data_to_list(self.annotation_df.columns)
         self.attributes_set = parsing.data_to_set(self.attributes)
 
+    def plot_results(self) -> plt.Figure:
+        fig = super().plot_results()
+
+        if self.plot_pathway_graphs:
+            for pathway in self.pathway_names_dict:
+                if pathway in self.results.index and self.results.loc[pathway, 'significant']:
+                    status = self.pathway_plot(pathway)
+                    if not status:
+                        break
+
+        return fig
+
+    def pathway_plot(self, pathway_id: str) -> bool:
+        ylabel = self.SINGLE_SET_ENRICHMENT_SCORE_YLABEL if self.single_set else self.ENRICHMENT_SCORE_YLABEL
+        pathway_tree = ontology.fetch_kegg_pathway(pathway_id, self.gene_id_translator)
+        if self.single_set:
+            highlight_genes = self.gene_set
+            # TODO: color by rank/ranking metric?
+        else:
+            highlight_genes = self.gene_set
+        return pathway_tree.plot_pathway(highlight_genes, ylabel=ylabel, graph_format=self.pathway_graphs_format)
+
 
 class GOEnrichmentRunner(EnrichmentRunner):
     __slots__ = {'dag_tree': 'DAG tree containing the hierarchical structure of all GO Terms',
@@ -930,18 +994,18 @@ class GOEnrichmentRunner(EnrichmentRunner):
                  databases: Union[str, Iterable[str]], excluded_databases: Union[str, Iterable[str]],
                  qualifiers: Union[str, Iterable[str]], excluded_qualifiers: Union[str, Iterable[str]],
                  return_nonsignificant: bool, save_csv: bool, fname: str, return_fig: bool, plot_horizontal: bool,
-                 plot_ontology_graph: bool, set_name: str, parallel: bool, enrichment_func_name: str, biotypes=None,
-                 background_set: set = None, biotype_ref_path: str = None, single_set: bool = False,
-                 random_seed: int = None, ontology_graph_format='pdf', **pvalue_kwargs):
+                 plot_ontology_graph: bool, set_name: str, parallel_backend: Literal[PARALLEL_BACKENDS],
+                 enrichment_func_name: str, biotypes=None, background_set: set = None, biotype_ref_path: str = None,
+                 exclude_unannotated_genes: bool = True, single_set: bool = False, random_seed: int = None,
+                 ontology_graph_format: Literal[GRAPHVIZ_FORMATS] = 'none', **pvalue_kwargs):
 
         self.propagate_annotations = propagate_annotations.lower()
         super().__init__(genes, [], alpha, '', return_nonsignificant, save_csv, fname, return_fig, plot_horizontal,
-                         set_name, parallel,
-                         enrichment_func_name, biotypes, background_set, biotype_ref_path, single_set, random_seed,
-                         **pvalue_kwargs)
+                         set_name, parallel_backend, enrichment_func_name, biotypes, background_set, biotype_ref_path,
+                         exclude_unannotated_genes, single_set, random_seed, **pvalue_kwargs)
         if not self.enrichment_func:
             return
-        self.dag_tree: ontology.DAGTree = io.fetch_go_basic()
+        self.dag_tree: ontology.DAGTree = ontology.fetch_go_basic()
         self.mod_annotation_dfs: Tuple[pd.DataFrame, ...] = tuple()
         self.gene_id_type = gene_id_type
         self.taxon_id, self.organism = self.get_taxon_id(organism)
@@ -1081,9 +1145,10 @@ class GOEnrichmentRunner(EnrichmentRunner):
         n_bars = min(10, len(self.results))
         kwargs = dict()
         if self.single_set:
-            kwargs['ylabel'] = r"$\log_2$(XL-mHG enrichment score)"
+            kwargs['ylabel'] = self.SINGLE_SET_ENRICHMENT_SCORE_YLABEL
             kwargs['title'] = f"Single-list GO enrichment for {self.set_name}\ntop {n_bars} most specific GO terms"
         else:
+            kwargs['ylabel'] = self.ENRICHMENT_SCORE_YLABEL
             kwargs['title'] = f"GO enrichment for {self.set_name}\ntop {n_bars} most specific GO terms"
 
         bar_plot = self.enrichment_bar_plot(n_bars=n_bars, **kwargs)
@@ -1098,93 +1163,12 @@ class GOEnrichmentRunner(EnrichmentRunner):
         aspects = ('biological_process', 'cellular_component',
                    'molecular_function') if self.aspects == 'any' else parsing.data_to_tuple(self.aspects)
         for go_aspect in aspects:
-            status = self._dag_plot_for_namespace(go_aspect, title, ylabel, dpi=dpi)
+            this_title = title + f'\n{go_aspect}'.replace('_', ' ').title()
+            status = self.dag_tree.plot_ontology(go_aspect, self.results, self.en_score_col, this_title, ylabel,
+                                                 self.ontology_graph_format, dpi)
             if not status:
                 return
 
-    def _dag_plot_for_namespace(self, namespace: str, title: str, ylabel: str, dpi: int = 300) -> bool:
-        # colormap
-        scores_no_inf = [i for i in self.results[self.en_score_col] if i != np.inf and i != -np.inf and i < 0]
-        if len(scores_no_inf) == 0:
-            scores_no_inf.append(-1)
-        max_score = max(np.max(scores_no_inf), 2)
-        my_cmap = plt.cm.get_cmap('coolwarm')
-
-        # generate graph
-        graph = graphviz.Digraph()
-        graph.attr(dpi=str(dpi))
-        node_queue = queue.SimpleQueue()
-        processed = set()
-        significant = set(self.results[self.results['significant']].index)
-        for sig_node in significant:
-            node_queue.put(sig_node)
-
-        while not node_queue.empty():
-            this_node = node_queue.get()
-            kwargs = dict(shape='box', style='rounded')
-
-            if this_node in processed:
-                continue
-            if self.dag_tree[this_node].namespace != namespace:
-                continue
-            # color significant nodes according to their enrichment score
-            if this_node in significant:
-                this_score = self.results[self.en_score_col].loc[this_node]
-                color_norm = 0.5 * (1 + this_score / (np.floor(max_score) + 1)) * 255
-                color_norm_8bit = int(color_norm) if color_norm != np.inf and color_norm != -np.inf else np.sign(
-                    color_norm) * max(np.abs(scores_no_inf))
-                color = tuple([int(i * 255) for i in my_cmap(color_norm_8bit)[:-1]])
-                kwargs['fillcolor'] = '#%02x%02x%02x' % color
-                kwargs['style'] = 'rounded, filled'
-                if int(np.mean(color)) < 128:
-                    kwargs['fontcolor'] = '#ffffff'
-
-            graph.node(this_node[3::], label=self.dag_tree[this_node].name, **kwargs)
-            processed.add(this_node)
-            #  add relationships to all parent nodes, and add parent nodes to queue
-            for relationship_type in ('is_a', 'part_of'):
-                relationship_label = relationship_type.replace('_', ' ') + ':'
-                for parent in self.dag_tree[this_node].get_parents(relationship_type):
-                    if self.dag_tree[parent].namespace == namespace:
-                        node_queue.put(parent)
-                        graph.edge(parent[3::], this_node[3::], label=relationship_label)
-
-        # save and display graph
-        savepath = io.get_todays_cache_dir().joinpath(f'dag_tree_{namespace}.{self.ontology_graph_format}')
-        i = 0
-        while savepath.exists():
-            i += 1
-            savepath = io.get_todays_cache_dir().joinpath(f'dag_tree_{namespace}_{i}.{self.ontology_graph_format}')
-        try:
-            graph.render(savepath, view=True, format=self.ontology_graph_format)
-        except graphviz.backend.execute.ExecutableNotFound:
-            warnings.warn("You must install 'GraphViz' and add it to PATH in order to generate Ontology Graphs. \n"
-                          "Please see https://graphviz.org/download/ for more information. ")
-            return False
-        # show graph in a matplotlib window
-        png_str = graph.pipe(format='png')
-        sio = builtin_io.BytesIO()
-        sio.write(png_str)
-        sio.seek(0)
-        img = mpimg.imread(sio)
-        fig, ax = plt.subplots(figsize=(14, 9))
-        ax.imshow(img, aspect='equal')
-        ax.axis('off')
-        ax.set_title(title + f'\n{namespace}'.replace('_', ' ').title(), fontsize=24)
-
-        # determine bounds, and enlarge the bound by a small margin (0.2%) so nothing gets cut out of the figure
-        bounds = np.array([np.ceil(-max_score) - 1, (np.floor(max_score) + 1)]) * 1.002
-        # add colorbar
-        sm = ScalarMappable(cmap=my_cmap, norm=plt.Normalize(*bounds))
-        sm.set_array(np.array([]))
-        cbar_label_kwargs = dict(label=ylabel, fontsize=16, labelpad=15)
-        cbar = fig.colorbar(sm, ticks=range(int(bounds[0]), int(bounds[1]) + 1), location='bottom')
-        cbar.set_label(**cbar_label_kwargs)
-        cbar.ax.tick_params(labelsize=14, pad=6)
-
-        plt.show()
-        return True
-
     def format_results(self, unformatted_results_dict: dict):
         if self.single_set:
             columns = ['name', 'samples', self.en_score_col, 'pval']
@@ -1271,7 +1255,8 @@ class GOEnrichmentRunner(EnrichmentRunner):
     def _parallel_over_grouping(self, func, grouping: Iterable, mod_df_inds: Iterable[int],
                                 max_nbytes: Union[str, None] = '1M', progress_bar_desc: str = '') -> dict:
         assert validation.is_method_of_class(func, type(self))
-        result_dicts = generic.ProgressParallel(n_jobs=-1, max_nbytes=max_nbytes, desc=progress_bar_desc)(
+        result_dicts = generic.ProgressParallel(desc=progress_bar_desc, n_jobs=-1, max_nbytes=max_nbytes,
+                                                backend=self.parallel_backend)(
             joblib.delayed(func)(group, ind) for group, ind in zip(grouping, mod_df_inds))
         result = {}
         for d in result_dicts:
