@@ -1,10 +1,13 @@
 import concurrent.futures
 import functools
+import gzip
 import inspect
 import json
 import os
+import platform
 import queue
 import re
+import shlex
 import shutil
 import subprocess
 import time
@@ -15,6 +18,7 @@ from functools import lru_cache
 from io import BytesIO, StringIO
 from itertools import chain
 from pathlib import Path
+from sys import executable
 from typing import List, Set, Union, Iterable, Tuple, Dict, Any, Callable
 from urllib.parse import urlparse, parse_qs, urlencode
 
@@ -23,15 +27,15 @@ import numpy as np
 import pandas as pd
 import requests
 import yaml
+from defusedxml import ElementTree
 from requests.adapters import HTTPAdapter, Retry
 from tqdm.auto import tqdm
-from sys import executable
 
 try:
     from typing import Literal
 except ImportError:
     from typing_extensions import Literal
-from rnalysis.utils import parsing, validation, ontology, __path__
+from rnalysis.utils import parsing, validation, __path__
 from rnalysis import __version__
 
 
@@ -75,8 +79,8 @@ def cache_file(content: str, filename: str):
         f.write(content)
 
 
-def clear_gui_cache():
-    directory = get_gui_cache_dir()
+def clear_directory(directory: Union[str, Path]):
+    directory = Path(directory)
     if not directory.exists():
         return
 
@@ -87,6 +91,16 @@ def clear_gui_cache():
             shutil.rmtree(item, ignore_errors=True)
 
 
+def clear_cache():
+    cache_dir = Path(appdirs.user_cache_dir('RNAlysis'))
+    clear_directory(cache_dir)
+
+
+def clear_gui_cache():
+    directory = get_gui_cache_dir()
+    clear_directory(directory)
+
+
 def load_cached_gui_file(filename: str):
     directory = get_gui_cache_dir()
     file_path = directory.joinpath(filename)
@@ -280,6 +294,9 @@ class KEGGAnnotationIterator:
     REQUEST_DELAY_MILLIS = 250
     REQ_MAX_ENTRIES = 10
     TAXON_TREE_CACHED_FILENAME = 'kegg_taxon_tree.json'
+    PATHWAY_NAMES_CACHED_FILENAME = 'kegg_pathway_list.txt'
+    COMPOUND_LIST_CACHED_FILENAME = 'kegg_compound_list.txt'
+    GLYCAN_LIST_CACHED_FILENAME = 'kegg_glycan_list.txt'
 
     def __init__(self, taxon_id: int, pathways: Union[str, List[str], Literal['all']] = 'all'):
         self.pathway_names = {}
@@ -332,6 +349,23 @@ class KEGGAnnotationIterator:
             taxon_tree = json.loads(content)
         return taxon_tree
 
+    @staticmethod
+    def get_compounds() -> Dict[str, str]:
+        compounds = {}
+        data = KEGGAnnotationIterator._kegg_request('list', ['compound'],
+                                                    KEGGAnnotationIterator.COMPOUND_LIST_CACHED_FILENAME)[0] + '\n' + \
+               KEGGAnnotationIterator._kegg_request('list', ['glycan'],
+                                                    KEGGAnnotationIterator.GLYCAN_LIST_CACHED_FILENAME)[0]
+        data = data.split('\n')
+        for line in data:
+            split = line.split('\t')
+            if len(split) == 2:
+                pathway_code, compound_names = split
+                main_name = compound_names.split(';')[0]
+                compounds[pathway_code] = main_name
+
+        return compounds
+
     @staticmethod
     @functools.lru_cache(1024)
     def get_kegg_organism_code(taxon_id: int) -> str:
@@ -352,7 +386,7 @@ class KEGGAnnotationIterator:
 
     def get_pathways(self) -> Tuple[Dict[str, str], int]:
         pathway_names = {}
-        data, _ = self._kegg_request('list', ['pathway', self.organism_code])
+        data, _ = self._kegg_request('list', ['pathway', self.organism_code], self.PATHWAY_NAMES_CACHED_FILENAME)
         data = data.split('\n')
         for line in data:
             split = line.split('\t')
@@ -363,6 +397,13 @@ class KEGGAnnotationIterator:
         n_annotations = len(pathway_names)
         return pathway_names, n_annotations
 
+    @staticmethod
+    def get_pathway_kgml(pathway_id: str) -> ElementTree:
+        cached_filename = f'kgml_{pathway_id}.xml'
+        data, _ = KEGGAnnotationIterator._kegg_request('get', [pathway_id, 'kgml'], cached_filename)
+        cache_file(data, cached_filename)
+        return ElementTree.parse(StringIO(data))
+
     def get_pathway_annotations(self):
         if self.pathway_annotations is not None:
             for pathway, annotations in self.pathway_annotations.items():
@@ -681,6 +722,13 @@ class GOlrAnnotationIterator:
         return self._annotation_generator_func()
 
 
+def get_obo_basic_stream():
+    url = 'http://current.geneontology.org/ontology/go-basic.obo'
+    with requests.get(url, stream=True) as obo_stream:
+        content = obo_stream.content.decode('utf8')
+        return content
+
+
 # TODO: cache this! save and load gene IDs individually
 @lru_cache(maxsize=32, typed=False)
 def _ensmbl_lookup_post_request(gene_ids: Tuple[str]) -> Dict[str, Dict[str, Any]]:
@@ -984,7 +1032,7 @@ def map_gene_ids(ids: Union[str, Iterable[str]], map_from: str, map_to: str = 'U
 
     # if map_from and map_to are the same, return an empty GeneIDTranslator (which will map any given gene ID to itself)
     id_dicts = _get_id_abbreviation_dicts()
-    validation.validate_uniprot_dataset_name(id_dicts, map_to, map_from)
+    validation.validate_uniprot_dataset_name(id_dicts, parsing.data_to_list(map_to), parsing.data_to_list(map_from))
     id_dict_to, id_dict_from = id_dicts
     if id_dict_to[map_to] == id_dict_from[map_from]:
         return GeneIDTranslator()
@@ -1192,7 +1240,8 @@ def get_legal_gene_id_types():
 
 
 @functools.lru_cache(maxsize=2)
-def _get_id_abbreviation_dicts(dict_path: str = os.path.join(__path__[0], 'uniprot_dataset_abbreviation_dict.json')):
+def _get_id_abbreviation_dicts(dict_path: str = os.path.join(__path__[0],
+                                                             '../data_files/uniprot_dataset_abbreviation_dict.json')):
     with open(dict_path) as f:
         abbrev_dict_to = json.load(f)
         abbrev_dict_from = abbrev_dict_to.copy()
@@ -1210,30 +1259,6 @@ def _get_id_abbreviation_dicts(dict_path: str = os.path.join(__path__[0], 'unipr
     return abbrev_dict_to, abbrev_dict_from
 
 
-@lru_cache(maxsize=2)
-def fetch_go_basic() -> ontology.DAGTree:
-    """
-    Fetches the basic Gene Ontology OBO file from the geneontology.org website ('go-basic.obo') and parses it into a \
-    DAGTree data structure.
-    :return: a parsed DAGTree for gene ontology propagation and visualization.
-    :rtype: utils.ontology.DAGTree
-    """
-    url = 'http://current.geneontology.org/ontology/go-basic.obo'
-    cached_filename = 'go-basic.obo'
-    cached_file = load_cached_file(cached_filename)
-    if cached_file is not None:
-        lines = cached_file.split('\n')
-        try:
-            return ontology.DAGTree(lines)
-        except (ValueError, IndexError):
-            pass
-
-    with requests.get(url, stream=True) as obo_stream:
-        content = obo_stream.content.decode('utf8')
-        cache_file(content, cached_filename)
-        return ontology.DAGTree(content.split('\n'))
-
-
 def get_datetime():
     now = datetime.now()
     now_str = now.strftime('%H:%M:%S %Y/%m/%d')
@@ -1279,7 +1304,14 @@ def run_r_script(script_path: Union[str, Path], r_installation_folder: Union[str
 
 
 def run_subprocess(args: List[str], print_stdout: bool = True, print_stderr: bool = True,
-                   log_filename: Union[str, None] = None):
+                   log_filename: Union[str, None] = None, shell: bool = False):
+    # join List of args into a string of args when running in shell mode
+    if shell:
+        try:
+            args = shlex.join(args)
+        except AttributeError:
+            args = ' '.join([shlex.quote(arg) for arg in args])
+
     if print_stdout or print_stderr:
         if print_stdout and print_stderr:
             stdout = subprocess.PIPE
@@ -1291,7 +1323,7 @@ def run_subprocess(args: List[str], print_stdout: bool = True, print_stderr: boo
             stdout = subprocess.DEVNULL
             stderr = subprocess.PIPE
 
-        with subprocess.Popen(args, stdout=stdout, stderr=stderr) as process:
+        with subprocess.Popen(args, stdout=stdout, stderr=stderr, shell=shell) as process:
             stream = process.stdout if print_stdout else process.stderr
             if log_filename is not None:
                 with open(log_filename, 'w') as logfile:
@@ -1305,7 +1337,7 @@ def run_subprocess(args: List[str], print_stdout: bool = True, print_stderr: boo
                     print(text)
         return_code = process.returncode
     else:
-        return_code = subprocess.run(args, stdout=subprocess.DEVNULL, stderr=subprocess.DEVNULL).returncode
+        return_code = subprocess.run(args, stdout=subprocess.DEVNULL, stderr=subprocess.DEVNULL, shell=shell).returncode
 
     return return_code
 
@@ -1423,3 +1455,32 @@ def map_transcripts_to_genes(gtf_path: Union[str, Path], use_name: bool = False,
 
                 mapping[transcript_id] = gene_name if use_name else gene_id
     return mapping
+
+
+def generate_base_call(command: str, installation_folder: Union[str, Path, Literal['auto']],
+                       version_command: str = '--version', args=tuple(), shell: bool = False):
+    if installation_folder == 'auto':
+        call = [command] + parsing.data_to_list(args)
+    else:
+        installation_folder = Path(installation_folder)
+        call = [installation_folder.joinpath(command).as_posix()] + parsing.data_to_list(args)
+
+    try:
+        exit_code = run_subprocess(call + [version_command], shell=shell)
+        assert exit_code == 0, f"call to {call[0]} exited with exit status {exit_code}."
+    except FileNotFoundError:
+        raise FileNotFoundError(f"RNAlysis could not find '{command}'. "
+                                'Please ensure that your installation folder is correct, or add it to PATH. ')
+
+    return call
+
+
+def get_gunzip_size(fn):
+    size = 0
+    with gzip.open(fn) as f:
+        while True:
+            data = f.read(8192)
+            size += len(data)
+            if not data:
+                break
+    return size
