@@ -179,15 +179,6 @@ def test_biotypes():
     assert np.all(df == truth)
 
 
-def tests_enrichment_randomization_api():
-    genes = {'WBGene00048865', 'WBGene00000864', 'WBGene00000105', 'WBGene00001996', 'WBGene00011910', 'WBGene00268195'}
-    attrs = ['attribute1', 'attribute2']
-    en = FeatureSet(gene_set=genes, set_name='test_set')
-    _ = en.enrich_randomization(attrs, reps=1, biotype='all', attr_ref_path=__attr_ref__,
-                                biotype_ref_path=__biotype_ref__)
-    plt.close('all')
-
-
 def test_enrichment_randomization_reliability():
     genes = {'WBGene00000041', 'WBGene00002074', 'WBGene00000019', 'WBGene00000105', 'WBGene00000106', 'WBGene00199484',
              'WBGene00001436', 'WBGene00000137', 'WBGene00001996', 'WBGene00014208'}
@@ -195,22 +186,22 @@ def test_enrichment_randomization_reliability():
     en = FeatureSet(gene_set=genes, set_name='test_set')
     random_seed = 0
 
-    for i in range(3):
+    for i in range(2):
         res1 = en.user_defined_enrichment(attrs, statistical_test='randomization', randomization_reps=10000,
                                           biotype='all',
                                           attr_ref_path=__attr_ref__,
                                           biotype_ref_path=__biotype_ref__,
-                                          random_seed=random_seed, parallel=False)
+                                          random_seed=random_seed, parallel_backend='sequential')
         res2 = en.user_defined_enrichment(attrs, statistical_test='randomization', randomization_reps=10000,
                                           biotype='all',
                                           attr_ref_path=__attr_ref__,
                                           biotype_ref_path=__biotype_ref__,
-                                          random_seed=random_seed + 1, parallel=False)
+                                          random_seed=random_seed + 1, parallel_backend='sequential')
         res3 = en.user_defined_enrichment(attrs, statistical_test='randomization', randomization_reps=10000,
                                           biotype='all',
                                           attr_ref_path=__attr_ref__,
                                           biotype_ref_path=__biotype_ref__,
-                                          random_seed=random_seed + 2, parallel=False)
+                                          random_seed=random_seed + 2, parallel_backend='sequential')
         random_seed += 3
         plt.close('all')
         for col in ['samples', 'obs', 'exp', 'log2_fold_enrichment']:
@@ -249,7 +240,7 @@ def test_enrichment_randomization_validity():
     en = FeatureSet(gene_set=genes, set_name='test_set')
     res = en.user_defined_enrichment(attrs, statistical_test='randomization', randomization_reps=100000, biotype='all',
                                      attr_ref_path=__attr_ref__,
-                                     biotype_ref_path=__biotype_ref__, random_seed=0, parallel=False)
+                                     biotype_ref_path=__biotype_ref__, random_seed=0, parallel_backend='sequential')
     plt.close('all')
     _enrichment_validity(res, truth)
 
@@ -261,22 +252,22 @@ def test_enrichment_randomization_parallel_reliability():
     en = FeatureSet(gene_set=genes, set_name='test_set')
     random_seed = 0
 
-    for i in range(3):
-        res1 = en.user_defined_enrichment(attrs, statistical_test='randomization', randomization_reps=10000,
+    for i in range(2):
+        res1 = en.user_defined_enrichment(attrs, statistical_test='randomization', randomization_reps=5000,
                                           biotype='all',
                                           attr_ref_path=__attr_ref__,
                                           biotype_ref_path=__biotype_ref__,
-                                          random_seed=random_seed, parallel=True)
-        res2 = en.user_defined_enrichment(attrs, statistical_test='randomization', randomization_reps=10000,
+                                          random_seed=random_seed, parallel_backend='loky')
+        res2 = en.user_defined_enrichment(attrs, statistical_test='randomization', randomization_reps=5000,
                                           biotype='all',
                                           attr_ref_path=__attr_ref__,
                                           biotype_ref_path=__biotype_ref__,
-                                          random_seed=random_seed + 1, parallel=True)
-        res3 = en.user_defined_enrichment(attrs, statistical_test='randomization', randomization_reps=10000,
+                                          random_seed=random_seed + 1, parallel_backend='multiprocessing')
+        res3 = en.user_defined_enrichment(attrs, statistical_test='randomization', randomization_reps=5000,
                                           biotype='all',
                                           attr_ref_path=__attr_ref__,
                                           biotype_ref_path=__biotype_ref__,
-                                          random_seed=random_seed + 2, parallel=True)
+                                          random_seed=random_seed + 2, parallel_backend='threading')
         random_seed += 3
         plt.close('all')
         for col in ['samples', 'obs', 'exp', 'log2_fold_enrichment']:
@@ -297,19 +288,11 @@ def test_enrichment_parallel_validity():
     en = FeatureSet(gene_set=genes, set_name='test_set')
     res = en.user_defined_enrichment(attrs, statistical_test='randomization', randomization_reps=100000, biotype='all',
                                      attr_ref_path=__attr_ref__,
-                                     biotype_ref_path=__biotype_ref__, random_seed=0, parallel=True)
+                                     biotype_ref_path=__biotype_ref__, random_seed=0, parallel_backend="loky")
     plt.close('all')
     _enrichment_validity(res, truth)
 
 
-def test_enrich_hypergeometric_api():
-    genes = {'WBGene00048865', 'WBGene00000864', 'WBGene00000105', 'WBGene00001996', 'WBGene00011910', 'WBGene00268195'}
-    attrs = ['attribute1', 'attribute2']
-    en = FeatureSet(gene_set=genes, set_name='test_set')
-    _ = en.enrich_hypergeometric(attrs, biotype='all', attr_ref_path=__attr_ref__, biotype_ref_path=__biotype_ref__)
-    plt.close('all')
-
-
 def test_enrich_hypergeometric_pvalues():
     truth = io.load_csv('tests/test_files/enrichment_hypergeometric_res.csv', 0)
     genes = {'WBGene00000041', 'WBGene00002074', 'WBGene00000105', 'WBGene00000106', 'WBGene00199484',
@@ -352,10 +335,10 @@ def test_featureset_repr():
     assert repr(en) == "FeatureSet: 'my very important set'"
 
 
-def test_plot_enrichment_results():
-    df = pd.read_csv('tests/test_files/enrichment_hypergeometric_res.csv')
-    plot_enrichment_results(df, plot_horizontal=True)
-    plot_enrichment_results(df, plot_horizontal=False, ylabel='different ylabel', alpha=0.1, name_col='name')
+def test_enrichment_bar_plot():
+    pth = 'tests/test_files/enrichment_hypergeometric_res.csv'
+    enrichment_bar_plot(pth, plot_horizontal=True)
+    enrichment_bar_plot(pth, plot_horizontal=False, ylabel='different ylabel', alpha=0.1)
     plt.close('all')
 
 
@@ -542,15 +525,14 @@ def test_go_enrichment_single_set_api(organism, propagate_annotations):
                     'WBGene00004920', 'WBGene00011910', 'WBGene00014208']
 
     en = RankedSet(genes_ranked, set_name='test_set')
-    _ = en.single_set_go_enrichment(organism, 'WBGene', evidence_types='experimental',
+    _ = en.single_set_go_enrichment(organism, 'WBGene', evidence_types='experimental', databases='WB',
                                     aspects='biological_process', propagate_annotations=propagate_annotations)
     plt.close('all')
 
 
 @pytest.mark.skipif(not ENSEMBL_AVAILABLE, reason='Ensembl REST API is not available at the moment')
 @pytest.mark.parametrize("organism,statistical_test,propagate_annotations,kwargs",
-                         [('auto', 'hypergeometric', 'classic', {}),
-                          ('auto', 'fisher', 'elim', {}),
+                         [('auto', 'fisher', 'elim', {}),
                           ('caenorhabditis elegans', 'randomization', 'no', dict(randomization_reps=100))])
 def test_go_enrichment_api(organism, statistical_test, propagate_annotations, kwargs):
     genes = {'WBGene00048865', 'WBGene00000864', 'WBGene00000105', 'WBGene00001996', 'WBGene00011910', 'WBGene00268195'}
@@ -632,5 +614,5 @@ def test_kegg_enrichment_api(organism, statistical_test, kwargs):
 def test_kegg_enrichment_single_list_api():
     genes = ['WBGene00048865', 'WBGene00000864', 'WBGene00000105', 'WBGene00001996', 'WBGene00011910', 'WBGene00268195']
     en = RankedSet(genes, set_name='test_set')
-    _ = en.single_set_kegg_enrichment(6239, 'WormBase')
+    _ = en.single_set_kegg_enrichment(6239, 'WormBase',pathway_graphs_format='pdf')
     plt.close('all')
