@@ -1,14 +1,8 @@
-import asyncio.log
-
-import pytest
-import pandas as pd
-from collections import namedtuple
-from sklearn.preprocessing import PowerTransformer, StandardScaler
-
 import matplotlib
+import pytest
 
+from rnalysis import filtering
 from rnalysis.filtering import *
-import os
 from tests import __attr_ref__, __biotype_ref__
 
 matplotlib.use('Agg')
@@ -16,19 +10,19 @@ matplotlib.use('Agg')
 
 def test_filter_api():
     f = Filter('tests/test_files/uncounted.csv')
-    assert f.__str__() == "Filter of file uncounted.csv"
+    assert f.__str__() == "Generic table named 'uncounted.csv'"
     assert f.__repr__().replace('\\', '/') == "Filter('tests/test_files/uncounted.csv')"
 
 
 def test_countfilter_api():
     h = CountFilter('tests/test_files/counted.csv')
-    assert h.__str__() == "CountFilter of file counted.csv"
+    assert h.__str__() == "Count matrix named 'counted.csv'"
     assert h.__repr__().replace('\\', '/') == "CountFilter('tests/test_files/counted.csv')"
 
 
 def test_deseqfilter_api():
     d = DESeqFilter('tests/test_files/test_deseq.csv')
-    assert d.__str__() == "DESeqFilter of file test_deseq.csv"
+    assert d.__str__() == "Differential expression table named 'test_deseq.csv'"
     assert d.__repr__().replace('\\', '/') == "DESeqFilter('tests/test_files/test_deseq.csv')"
 
 
@@ -1222,12 +1216,12 @@ def test_pipeline_repr():
 
 def test_pipeline_str():
     pl = Pipeline('countfilter')
-    assert str(pl) == "Pipeline for CountFilter objects"
+    assert str(pl) == "Pipeline for Count matrix"
     pl.add_function('sort')
     pl.add_function(CountFilter.filter_biotype_from_ref_table, 'protein_coding', opposite=True)
-    assert str(pl) == "Pipeline for CountFilter objects:\n" \
-                      "\tCountFilter.sort()\n" \
-                      "\tCountFilter.filter_biotype_from_ref_table('protein_coding', opposite=True)"
+    assert str(pl) == "Pipeline for Count matrix:\n" \
+                      "\tSort table rows: ()\n" \
+                      "\tFilter by feature biotype (based on a reference table): ('protein_coding', opposite=True)"
 
 
 def test_pipeline_add_function():
@@ -1503,7 +1497,7 @@ def test_pipeline_apply_to_filter_normalize_split_plot():
     pl_c = Pipeline('CountFilter')
     pl_c.add_function(CountFilter.normalize_with_scaling_factors, scaling_factor_path)
     pl_c.add_function('biotypes_from_ref_table', ref=__biotype_ref__)
-    pl_c.add_function(CountFilter.filter_top_n, by='cond3rep1', n=17500, opposite=True)
+    pl_c.add_function(CountFilter.filter_top_n, by='cond3rep1', n=5000, opposite=True)
     pl_c.add_function(CountFilter.split_hdbscan, min_cluster_size=40, return_probabilities=True)
     pl_c.add_function(CountFilter.filter_low_reads, threshold=10)
     pl_c.add_function(CountFilter.clustergram)
@@ -1516,7 +1510,7 @@ def test_pipeline_apply_to_filter_normalize_split_plot():
     c_dict = dict()
     c.normalize_with_scaling_factors(scaling_factor_path)
     c_dict['biotypes_from_ref_table_1'] = c.biotypes_from_ref_table(ref=__biotype_ref__)
-    c_res = c.filter_top_n(by='cond3rep1', n=17500, opposite=True, inplace=False)
+    c_res = c.filter_top_n(by='cond3rep1', n=5000, opposite=True, inplace=False)
     c_res, prob = c_res.split_hdbscan(min_cluster_size=40, return_probabilities=True)
     c_dict['split_hdbscan_1'] = prob
     for obj in c_res:
@@ -1552,7 +1546,7 @@ def test_pipeline_apply_to_filter_normalize_split_plot():
 
 def test_split_kmeans_api():
     c = CountFilter('tests/test_files/big_counted.csv')
-    res = c.split_kmeans(n_clusters=4)
+    res = c.split_kmeans(n_clusters=4, n_init=2, max_iter=50)
     assert isinstance(res, tuple)
     assert len(res) == 4
     _test_correct_clustering_split(c, res)
@@ -1585,7 +1579,7 @@ def test_split_hierarchical_api():
 
 
 def test_split_hdbscan_api():
-    c = CountFilter('tests/test_files/big_counted.csv')
+    c = CountFilter('tests/test_files/big_counted.csv').filter_top_n('cond1rep1',n=2000,inplace=False)
     res = c.split_hdbscan(100)
     _test_correct_clustering_split(c, res, True)
     res2 = c.split_hdbscan(4, 5, 'manhattan', 0.2, 'leaf', plot_style='std_area', return_probabilities=True)
@@ -1775,7 +1769,7 @@ def test_import_pipeline():
 
 def test_export_pipeline():
     with open('tests/test_files/test_pipeline.yaml') as f:
-        truth = f.read()
+        truth = yaml.safe_load(f)
 
     p = Pipeline('countfilter')
     p.add_function(CountFilter.biotypes_from_ref_table)
@@ -1785,12 +1779,35 @@ def test_export_pipeline():
 
     try:
         with open(outname) as f:
-            exported = f.read()
-        assert exported == truth
+            exported = yaml.safe_load(f)
+        assert set(exported.keys()) == set(truth.keys())
+        for key in exported.keys():
+            if key == 'metadata':
+                assert set(exported[key].keys()) == set(truth[key].keys())
+                for internal_key in exported[key]:
+                    if internal_key == 'export_time':
+                        continue
+                    elif internal_key == 'rnalysis_version':
+                        assert exported[key][internal_key] == filtering.__version__
+            else:
+                assert exported[key] == truth[key]
     finally:
         os.remove(outname)
 
 
+@pytest.mark.parametrize('components,gene_fraction,truth_paths', [
+    (1, 0.32, ['tests/test_files/counted_pc1_0.32_top.csv', 'tests/test_files/counted_pc1_0.32_bottom.csv'])
+])
+def test_filter_by_principal_components(components, gene_fraction, truth_paths):
+    truth = [CountFilter(pth) for pth in truth_paths]
+    c = CountFilter('tests/test_files/counted.csv')
+    c.filter_low_reads(1)
+    res = c.split_by_principal_components(components, gene_fraction)
+    assert len(res) == len(truth)
+    for i in range(len(truth)):
+        assert res[i].df.sort_index().equals(truth[i].df.sort_index())
+
+
 @pytest.mark.parametrize('ids,mode,truth_path', [
     ('kegg_id2', 'union', 'tests/test_files/counted_filter_by_kegg_truth_1.csv'),
     (['kegg_id2'], 'intersection', 'tests/test_files/counted_filter_by_kegg_truth_1.csv'),
@@ -1871,7 +1888,7 @@ def test_filter_by_go_annotations(monkeypatch, ids, mode, truth_path):
 
     monkeypatch.setattr(io.GOlrAnnotationIterator, '_get_n_annotations', lambda *args, **kwargs: 13)
     monkeypatch.setattr(io.GOlrAnnotationIterator, '_annotation_generator_func', annotation_iter)
-    monkeypatch.setattr(io, 'fetch_go_basic', lambda: MockDAGTree())
+    monkeypatch.setattr(ontology, 'fetch_go_basic', lambda: MockDAGTree())
 
     f = Filter('tests/test_files/counted.csv')
     res = f.filter_by_go_annotations(ids, mode, gene_id_type='WormBase', inplace=False)
@@ -1882,3 +1899,14 @@ def test_filter_by_go_annotations(monkeypatch, ids, mode, truth_path):
         print(res.df)
         print(truth)
         raise e
+
+
+@pytest.mark.parametrize("pth,cols,truth_pth", [
+    ('tests/test_files/counted.csv', 'cond2', 'tests/test_files/counted_drop_columns_truth_1.csv'),
+    ('tests/test_files/counted.csv', ['cond4', 'cond2'], 'tests/test_files/counted_drop_columns_truth_2.csv'),
+])
+def test_drop_columns(pth, cols, truth_pth):
+    obj = Filter(pth)
+    truth = Filter(truth_pth)
+    obj.drop_columns(cols)
+    assert obj.df.sort_index().equals(truth.df.sort_index())
