@@ -1,5 +1,3 @@
-import os
-
 import pytest
 
 import rnalysis.utils.io
@@ -83,8 +81,8 @@ def test_save_csv():
             pass
 
 
-def test_fetch_go_basic_connectivity():
-    _ = fetch_go_basic()
+def test_obo_basic_stream_connectivity():
+    _ = get_obo_basic_stream()
 
 
 def test_format_ids_iter():
@@ -607,11 +605,13 @@ def test_kegg_annotation_iterator_get_pathways(monkeypatch):
     truth = (PATHWAY_NAMES_TRUTH, 6)
     organism_code = 'cel'
 
-    def mock_kegg_request(self, operation, arguments):
+    def mock_kegg_request(self, operation, arguments, cached_filename=None):
         assert operation == 'list'
         assert arguments == ['pathway', organism_code]
+        assert cached_filename == KEGGAnnotationIterator.PATHWAY_NAMES_CACHED_FILENAME
+
         with open('tests/test_files/kegg_pathways.txt') as f:
-            return f.read(), False
+            return f.read(), True
 
     monkeypatch.setattr(KEGGAnnotationIterator, '_kegg_request', mock_kegg_request)
 
@@ -620,6 +620,68 @@ def test_kegg_annotation_iterator_get_pathways(monkeypatch):
     assert kegg.get_pathways() == truth
 
 
+def test_kegg_annotation_iterator_get_compounds(monkeypatch):
+    reqs_made = []
+    truth = {'cpd:C00001': 'H2O',
+             'cpd:C00002': 'ATP',
+             'cpd:C00003': 'NAD+',
+             'cpd:C00004': 'NADH',
+             'cpd:C00005': 'NADPH',
+             'cpd:C00006': 'NADP+',
+             'cpd:C00007': 'Oxygen',
+             'cpd:C00008': 'ADP',
+             'cpd:C00009': 'Orthophosphate',
+             'cpd:C00010': 'CoA',
+             'cpd:C00011': 'CO2',
+             'cpd:C00012': 'Peptide', }
+
+    def mock_kegg_request(operation, arguments, cached_filename=None):
+        assert operation == 'list'
+        assert len(arguments) == 1
+        if arguments[0] == 'compound':
+            assert cached_filename == KEGGAnnotationIterator.COMPOUND_LIST_CACHED_FILENAME
+        else:
+            assert cached_filename == KEGGAnnotationIterator.GLYCAN_LIST_CACHED_FILENAME
+        reqs_made.append(arguments[0])
+        with open('tests/test_files/kegg_compounds.txt') as f:
+            return f.read(), True
+
+    monkeypatch.setattr(KEGGAnnotationIterator, '_kegg_request', mock_kegg_request)
+
+    kegg = KEGGAnnotationIterator.__new__(KEGGAnnotationIterator)
+    res = kegg.get_compounds()
+    assert res == truth
+    assert sorted(reqs_made) == ['compound', 'glycan']
+
+
+def are_xml_elements_equal(e1, e2):
+    if e1.tag != e2.tag: return False
+    if e1.text != e2.text: return False
+    if e1.tail != e2.tail: return False
+    if e1.attrib != e2.attrib: return False
+    if len(e1) != len(e2): return False
+    return all(are_xml_elements_equal(c1, c2) for c1, c2 in zip(e1, e2))
+
+
+@pytest.mark.parametrize('pathway_id,expected_fname', [('hsa:00001', 'kgml_hsa:00001.xml')])
+def test_kegg_annotation_iterator_get_pathway_kgml(monkeypatch, pathway_id, expected_fname):
+    pth = 'tests/test_files/test_kgml.xml'
+    with open(pth) as f:
+        truth = ElementTree.parse(f)
+
+    def mock_kegg_request(operation, arguments, cached_filename=None):
+        assert operation == 'get'
+        assert arguments == [pathway_id, 'kgml']
+        assert cached_filename == expected_fname
+        with open(pth) as f:
+            return f.read(), True
+
+    monkeypatch.setattr(KEGGAnnotationIterator, '_kegg_request', mock_kegg_request)
+
+    kegg = KEGGAnnotationIterator.__new__(KEGGAnnotationIterator)
+    assert are_xml_elements_equal(kegg.get_pathway_kgml(pathway_id).getroot(), truth.getroot())
+
+
 def test_kegg_annotation_iterator_get_custom_pathways(monkeypatch):
     truth = {'path:cel00010': None, 'path:cel00030': None, 'path:cel00051': None}
 
@@ -812,3 +874,51 @@ def test_is_rnalysis_outdated(monkeypatch, this_version, response, expected):
     monkeypatch.setattr(requests, 'get', lambda *args, **kwargs: response)
     monkeypatch.setattr(rnalysis.utils.io, '__version__', this_version)
     assert is_rnalysis_outdated() == expected
+
+
+@pytest.mark.parametrize('feature_type,truth', [
+    ('gene', {
+        'ENSG00000168671': 'protein_coding',
+        'ENSG00000249641': 'antisense_RNA',
+        'ENSG00000123364': 'protein_coding',
+        'ENSG00000123407': 'protein_coding',
+        'ENSG00000123388': 'protein_coding',
+        'ENSG00000180818': 'protein_coding',
+        'ENSG00000197757': 'protein_coding',
+        'ENSG00000180806': 'protein_coding',
+        'ENSG00000037965': 'protein_coding',
+        'ENSG00000198353': 'protein_coding',
+        'ENSG00000172789': 'other'}),
+    ('transcript', {
+        "ENST00000282507": "protein_coding",
+        "ENST00000513300": "protein_coding",
+        "ENST00000504685": "nonsense_mediated_decay",
+        "ENST00000504954": "processed_transcript",
+        "ENST00000515131": "protein_coding",
+        "ENST00000512916": "antisense_RNA",
+        "ENST00000243056": "protein_coding",
+        "ENST00000243103": "protein_coding",
+        "ENST00000546378": "protein_coding",
+        "ENST00000243082": "protein_coding",
+        "ENST00000515593": "protein_coding",
+        "ENST00000303460": "protein_coding",
+        "ENST00000511575": "processed_transcript",
+        "ENST00000514415": "processed_transcript",
+        "ENST00000504315": "protein_coding",
+        "ENST00000509328": "protein_coding",
+        "ENST00000394331": "protein_coding",
+        "ENST00000243108": "protein_coding",
+        "ENST00000513413": "processed_transcript",
+        "ENST00000504557": "processed_transcript",
+        "ENST00000508190": "protein_coding",
+        "ENST00000303450": "protein_coding",
+        "ENST00000040584": "protein_coding",
+        "ENST00000303406": "protein_coding",
+        "ENST00000507650": "processed_transcript",
+        "ENST00000430889": "protein_coding",
+        "ENST00000312492": "other", })
+])
+def test_map_gene_to_attr(feature_type, truth):
+    gtf_path = 'tests/test_files/kallisto_tests/transcripts.gtf'
+    res = map_gene_to_attr(gtf_path, feature_type + '_biotype', feature_type, False, False, False)
+    assert res == truth
